#! /bin/bash
# Bibliotheque pour les exercices du MOOC LI101
# On suppose que la solution est la pseudo-copie pseudos/perfect/FILE
# Quand ca va pas du tout, on peut ajouter, dans son script,
#   export FW4EX_SHOW_BIGLOO_OUTPUT=true
# afin de voir ce que Bigloo a fait. On peut aussi ajouter
#  export FW4EX_BIGLOO_DEBUG=true
# pour tracer les evaluations (mais ca peut faire beaucoup).
#  export FW4EX_BIGLOO_DEBUG_FROM=200
# pour ne montrer que les traces à partir de la 200eme evaluation.
#  export FW4EX_SHOW_SCHEME_FILE=false
# pour ne pas montrer le code Scheme de l'etudiant.

FW4EX_BIGLOO_DEBUG=${FW4EX_BIGLOO_DEBUG:-false}
FW4EX_SHOW_SCHEME_FILE=${FW4EX_SHOW_SCHEME_FILE:-true}
FW4EX_SHOW_BIGLOO_OUTPUT=${FW4EX_SHOW_BIGLOO_OUTPUT:-false}

fw4ex_handle () {
    # Un seul fichier .scm dans pseudos/perfect/ svp!
    local FILE="${1:-`cd $FW4EX_EXERCISE_DIR/pseudos/perfect/ && ls *.scm`}"
    local ENVIRONMENT="${2:-*the-environment*}"

    # Ajout du suffixe .scm s'il manque:
    case "$FILE" in 
        *.scm)
            true
            ;;
        *)
            FILE=$FILE.scm
            ;;
    esac

# pre-bigloo is the bigloo implementation specifics required by li101.scm
# li101.scm is the instrumented Scheme interpreter and the test harness
# li101+.scm is loaded if present and mostly used to change the weights
# of the marking process.

    #if $FW4EX_BIGLOO_DEBUG 
    #then FW4EX_SHOW_BIGLOO_OUTPUT=true
    #fi
    
    if $FW4EX_SHOW_SCHEME_FILE
    then
        fw4ex_li101_show_scheme_file "$FILE"
    fi
    echo -n > $TMPDIR/li101a.out
    echo -n > $TMPDIR/li101b.out
    echo -n > $TMPDIR/li101c.out
    echo 0 > $TMPDIR/li101a.mark
    echo 0 > $TMPDIR/li101b.mark
    {
        cat <<EOF
(load "$FW4EX_EXERCISE_DIR/pre-bigloo.scm")
(load "$FW4EX_EXERCISE_DIR/li101.scm")
$( if [ -f $FW4EX_EXERCISE_DIR/li101+.scm ] ; 
   then echo "(load \"$FW4EX_EXERCISE_DIR/li101+.scm\")" ; fi )
$( if $FW4EX_BIGLOO_DEBUG 
   then echo '(set! *debug* #t)' 
   fi )
(li101-process "$FILE" 
               "$FW4EX_EXERCISE_DIR/pseudos/perfect/$FILE"
               $ENVIRONMENT
               "$TMPDIR" )
EOF
    } | fw4ex_confine bigloo 2>$TMPDIR/li101.err 1>&2
    fw4ex_generate_xml_comment exitCode $( cat $TMPDIR/.lastExitCode )
    fw4ex_verbalize_error_code $TMPDIR/.lastExitCode

    if [ -f $TMPDIR/li101img.eps ]
    then 
        echo "<p>Voici l'image que vous avez créée: </p>"
        convert $TMPDIR/li101img.eps $TMPDIR/li101img.png
        fw4ex_insert_image $TMPDIR/li101img.png
        cat <<EOF
<p>Vous gagnez $( fw4ex_win 1 ) point. </p>
EOF
    else 
        echo "<p>Voici ce que produit la correction automatisée:</p>"
        # li101?.out doivent contenir du XML correct (balises et codage):
        echo "<section name='correction'>"
        cat $TMPDIR/li101a.out 
        if [ -f $TMPDIR/li101-.err ]
        then
            echo
            echo "<FW4EX what='Internal error'/>"
            echo "<error>Problème interne lors de la correction.</error>"
            cat $TMPDIR/li101-.err >&4
            if $FW4EX_BIGLOO_DEBUG
            then
                fw4ex_generate_xml_multiline_comment $TMPDIR/li101-.err
            fi
        fi
        cat <<EOF
<p>Vous gagnez $( fw4ex_win $(cat $TMPDIR/li101a.mark) ) points. </p>
EOF
        echo "</section>"
        if [ -s $TMPDIR/li101b.out ]
        then
            echo "<section name='couverture'>"
            cat $TMPDIR/li101b.out 
            cat <<EOF
<p>Vous gagnez encore $( fw4ex_win $(cat $TMPDIR/li101b.mark) ) points. </p>
EOF
            echo "</section>"
        fi
        if [ -s $TMPDIR/li101c.out ]
        then
            echo "<section name='complexité'>"
            cat $TMPDIR/li101c.out
            if [ -s $TMPDIR/li101c.mark ]
            then
                cat <<EOF
<error> En conclusion, un malus de $( fw4ex_win $(cat $TMPDIR/li101c.mark) ) 
points vous est également attribué. </error>
EOF
            fi
            echo "</section>"
        fi
    fi

    if ${FW4EX_SHOW_BIGLOO_OUTPUT}
    then
        echo "<p> Bigloo a aussi produit: <pre>"
        if [ -z "${FW4EX_BIGLOO_DEBUG_FROM}" ]
        then
            fw4ex_transcode_carefully < $TMPDIR/li101.err
        else
            sed -nre "/^.li101-eval .${FW4EX_BIGLOO_DEBUG_FROM}. /,\$p" \
                $TMPDIR/li101.err | fw4ex_transcode_carefully 
        fi
        echo "</pre></p>"
    fi

    if [ -f $TMPDIR/li101.details ]
    then
        perl -pe 's/ (\d+) / $1\n/g' \
            <$TMPDIR/li101.details >$TMPDIR/li101.details2
        fw4ex_generate_xml_multiline_comment $TMPDIR/li101.details2
    else 
        fw4ex_generate_xml_comment "No details!?"
    fi
    if [ -f $TMPDIR/li101.profiling ]
    then
        fw4ex_generate_xml_multiline_comment $TMPDIR/li101.profiling
    else
        fw4ex_generate_xml_comment "No profiling!?"
    fi
}

fw4ex_li101_show_scheme_file () {
    local SCRIPT="$1"
    echo "<FW4EX phase='begin' what='SchemeCode' />"
    #while the new fw4ex.rnc is not everywhere:
    #echo "<p> Voici votre réponse: <pre id='fw4ex_student_code'>"
    echo "<p> Voici votre réponse: <pre>"
    fw4ex_transcode_carefully --l < "$SCRIPT"
    echo "</pre></p>"
    echo "<FW4EX phase='end' what='SchemeCode' />"
}

fw4ex_generate_xml_multiline_comment () {
    local FILE="$1"
    echo "<!--" 
    sed -r 's/-{2,}/-/g' < $FILE | fw4ex_transcode_carefully
    echo "-->"
}

fw4ex_li101_show_file () {
    local FILE="$1"
    if [[ 0 -eq $( wc -c < "$FILE" ) ]]
    then
        echo "<p><error>Votre fichier est vide!</error></p>"
    else
        case "$(file "$FILE")" in
          *text*)
            fw4ex_generate_fw4ex_element "file:///$FILE"
            cat <<EOF
<p>
Voici votre programme:
<FW4EX phase='begin' what='solution' />
<pre>
EOF
            fw4ex_transcode_carefully --l < "$FILE"
            cat <<EOF
</pre><FW4EX phase='end' what='solution' /></p>
EOF
            ;;
        *)
            cat <<EOF
<warning> Votre fichier 
<code>$( echo $FILE | fw4ex_transcode_carefully )</code> 
ne semble pas être un texte, 
l'utilitaire <code>file</code> dit en effet que c'est un
<code>$( file "$FILE" | fw4ex_transcode_carefully )</code>:
je ne tente donc pas de vous le montrer. </warning>
EOF
            ;;
        esac
    fi
}

# end of li101.sh
